@@ -65,7 +65,7 @@ is_prefix_op('#') -> numeric;
 is_prefix_op(_) -> false.
 
 check_regexp(String) ->
-    case re:compile(String) of
+    case re:compile(String, [unicode]) of
 	{ok, _Expr} ->
 	    {regexpr, String};
 	{error, {ErrString, Position}} ->
@@ -169,7 +169,7 @@ mfa2s({M,F,A}) ->
     [c2s(M),':',c2s(F),'/',A].
 
 c2s(C) ->
-    [S] = io_lib:format("~p", [C]),
+    [S] = io_lib:format("~tp", [C]),
     list_to_atom(S).
 
 re(variable) -> ['_'];
